#define FSR_PIN		(0)	// the FSR and 10K pulldown are connected to pin A0
#define TILT_PIN	(2)	// the tilt switch module is connected to pin 2

#define FSR_THRESHOLD_LOW	(10)
#define FSR_THRESHOLD_HIGH	(500)

#define SIGNAL_STOP			((int) 0)
#define SIGNAL_JUMP			((int) 1)
#define SIGNAL_DUCK_START	((int) 2)
#define SIGNAL_DUCK_END		((int) 3)

#define SERIAL_SPEED	(9600)
#define CLOCK_DELAY		(100)
#define JUMP_TIMEOUT	(2000)

#define DEBUG				(false)
#define SEND_STOP_SIGNAL	(false)

enum state {
	NONE,
	JUMPING,
	DUCKING,
	STOPPED
};

void setup() {
	Serial.begin(SERIAL_SPEED); // start the serial
}

void loop() {
	static int fsrReading; // the analog reading from the FSR resistor divider
	static int tiltState; // the digital reading from the tilt switch module
	
	static state currentState = NONE; // initially, no action is performed

	static unsigned long jumpStartTimestamp; // keep track of how long a jump takes

	// update the inputs
	fsrReading = analogRead(FSR_PIN);
	tiltState = digitalRead(TILT_PIN);

	// do actions according to the current state
	switch (currentState) {
		case NONE: {
			// wait for either a jump or a duck
			if (fsrReading < FSR_THRESHOLD_LOW) {
				// a low reading means the player is not touching the sensor
				jumpStartTimestamp = millis();
				currentState = JUMPING;
				Serial.print(SIGNAL_JUMP);
			} else if (tiltState == HIGH) {
				// a high reading means the player started ducking
				currentState = DUCKING;
				Serial.print(SIGNAL_DUCK_START);
			}

			break;
		}
		case JUMPING: {
			// wait for the player to return to the ground
			if (SEND_STOP_SIGNAL && millis() - jumpStartTimestamp > JUMP_TIMEOUT) {
				// instruct the script to end when the player is off the FSR for too long
				currentState = STOPPED;
				Serial.print(SIGNAL_STOP);
			}

			if (fsrReading > FSR_THRESHOLD_HIGH) {
				// a high reading means the player is touching the sensor
				currentState = NONE;
			}

			break;
		}
		case DUCKING: {
			// wait for the player to stand up again
			if (tiltState == LOW) {
				// a low reading means the player stopped ducking
				currentState = NONE;
				Serial.print(SIGNAL_DUCK_END);
			}

			break;
		}
		default: {
			// intentionally left empty
		}
	}

	if (DEBUG) {
		Serial.print("Current state = ");
		Serial.println(currentState);
		Serial.print("FSR: ");
		Serial.println(fsrReading);
		Serial.print("TSM: ");
		Serial.println(tiltState);
	}

	delay(CLOCK_DELAY);
}
