import serial  # to read from the USB serial
import pyautogui as keyboard  # for keyboard input

# set to True to enable logging
DEBUG = True

# operation codes
SIGNAL_STOP = b'0'
SIGNAL_JUMP = b'1'
SIGNAL_DUCK_START = b'2'
SIGNAL_DUCK_END = b'3'

# serial connection
SERIAL_PORT = 'COM3'
SERIAL_BAUDRATE = 9600
SERIAL_TIMEOUT = 1


# if the debug mode is activated, prints the specified message to stdout
def print_debug(message: str):
    if DEBUG:
        print(f"[DEBUG] {message}")


# initializes prerequisites (like the USB serial) and runs the loop waiting for input
def run_loop():
    # connect to the serial
    ser = serial.Serial(
        port=SERIAL_PORT,
        baudrate=SERIAL_BAUDRATE,
        timeout=SERIAL_TIMEOUT
    )

    def duck_start():
        print_debug("Started ducking!")
        keyboard.keyDown('down')

    def duck_end():
        print_debug("Stopped ducking!")
        keyboard.keyUp('down')

    def jump():
        print_debug("Jumped!")
        keyboard.press('space')

    # map every signal to the keyboard action
    serial_to_keyboard = {
        SIGNAL_DUCK_START: duck_start,
        SIGNAL_DUCK_END: duck_end,
        SIGNAL_JUMP: jump
    }

    print_debug("Started application...")

    while True:
        if ser.in_waiting == 0:
            continue  # nothing to read

        signal = ser.read(1)  # read a single byte from the serial

        print_debug(f"Read from Serial: {signal}")

        if signal == SIGNAL_STOP:
            print_debug("Stopped application...")
            break

        keyboard_action = serial_to_keyboard.get(signal)
        if not keyboard_action:
            print_debug(f"No action defined for this signal")
            continue

        keyboard_action()


if __name__ == '__main__':
    run_loop()
